import cognite.airworkflow.constants as const
from cognite.airworkflow.model.deployment import Deployment
from cognite.airworkflow.util import cdf, env, functions


class FunctionDeployer:
    def __init__(self, deployment: Deployment, is_delete: bool = False, pr: bool = False):
        project_info = const.PROJECTS_TO_API_KEYS[deployment.project]
        self.client = cdf.experimental_client(project_info.project)
        self.display_name = deployment.code_path.name
        self.function_name = functions.get_function_name(self.display_name, version=deployment.version, pr=pr)
        self.function_name_latest = functions.get_function_name(self.display_name, pr=pr, latest=True)
        self.file_name = self.function_name.replace("/", "_").replace(":", "_").replace(".", "_")

        self.is_delete = is_delete
        self.is_pr = pr
        self.deployment = deployment
        self.client_api_key = project_info.get_client_api_key()
        self.secrets = (
            {}
            if (self.is_delete or self.deployment.secret_names is None)
            else {s: env.get_env_value(s) for s in self.deployment.secret_names}
        )

    def handle_deployment(self):
        if self.is_pr:
            self.handle_pull_request()
        else:
            self.handle_push()

    def handle_pull_request(self):
        functions.delete_function(self.client, self.function_name)
        if self.is_delete:
            return

        file_id = cdf.zip_and_upload(
            self.client,
            self.deployment.code_path / const.FUNCTION_REL_PATH,
            self.file_name,
            "AIR",
        )
        functions.deploy_function(
            self.client,
            self.function_name,
            file_id,
            self.secrets,
            self.client_api_key,
            owner="AIR",
        )

    def handle_push(self) -> None:
        file_id = cdf.zip_and_upload(
            self.client,
            self.deployment.code_path / const.FUNCTION_REL_PATH,
            self.file_name,
            "AIR",
        )
        functions.deploy_function(
            self.client,
            self.function_name_latest,
            file_id,
            self.secrets,
            self.client_api_key,
            owner="AIR",
        )
        functions.delete_function(self.client, self.function_name)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': '.'}

packages = \
['cognite',
 'cognite.airworkflow',
 'cognite.airworkflow.model',
 'cognite.airworkflow.util']

package_data = \
{'': ['*'], 'cognite.airworkflow': ['schemas/*']}

install_requires = \
['PyGithub>=1.51,<2.0',
 'cerberus>=1.3.2,<2.0.0',
 'cognite-air-sdk>=2.5.2,<3.0.0',
 'cognite-sdk-experimental>=0,<1',
 'cognite-sdk>=2,<3',
 'croniter>=0.3.31,<0.4.0',
 'gitpython>=3.1.1,<4.0.0',
 'jinja2>=2.11.2,<3.0.0',
 'pyjwt>=1.7.1,<2.0.0',
 'pytest-custom_exit_code>=0.3.0,<0.4.0',
 'ruamel.yaml>=0.16.10,<0.17.0']

setup_kwargs = {
    'name': 'cognite-air-workflow',
    'version': '2.0.5',
    'description': 'Client library to perform all required cognite airflow functions built to function with AIR CDF',
    'long_description': None,
    'author': 'Arun Kaashyap Arunachalam',
    'author_email': 'arun.arunachalam@cognite.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import re
import shutil
from pathlib import Path
from typing import List


def read_file_to_list(path: Path) -> List[str]:
    return path.read_text().split("\n")


def get_file_paths(folder_path: Path, file_path: Path, ignore_dirs: List[str] = None) -> List[Path]:
    return [
        d / file_path
        for d in folder_path.iterdir()
        if d.is_dir() and (d / file_path).exists() and (ignore_dirs is not None and d.name not in ignore_dirs)
    ]


def sub_file_content(path: Path, regex: str, new_content: str) -> None:
    content = path.read_text()
    path.write_text(re.sub(regex, new_content, content))


def write(path: Path, content: str) -> None:
    path.write_text(content)


def copy_dir(src: Path, dest: Path) -> None:
    shutil.copytree(src, dest, dirs_exist_ok=True)  # type: ignore
